#!/usr/bin/env python

import sys
import os
import re

DEBUG=False

dir_mappings = {"../UI/Templates":"../UI/Common",
                "../UI/Templates/AdministrationUI":"../UI/AdministrationUI",
                "../UI/Templates/Appointments":"../SoObjects/Appointments",
                "../UI/Templates/ContactsUI":"../UI/Contacts",
                "../UI/Templates/MailerUI":"../UI/MailerUI",
                "../UI/Templates/MailPartViewers":"../UI/MailPartViewers",
                "../UI/Templates/MainUI":"../UI/MainUI",
                "../UI/Templates/PreferencesUI":"../UI/PreferencesUI",
                "../UI/Templates/SchedulerUI":"../UI/Scheduler"
               }

def get_translations(path):
    try:
        transpath = dir_mappings.get(path, path)
        transname = transpath + '/English.lproj/Localizable.strings'
        transall = open(transname).read()
    except:
        transall = ""
    return transall


def find_missing_translations(rootdir='.', extention='', recomp=None, greylist=()):
    for path, dirs, files in os.walk(rootdir):
        filelist = [f for f in files if f[(-1 * len(extention)):] == extention]
        if filelist:
            for filename in filelist:
                if filename in greylist:
                    print "%s is greylisted -- SKIPPING" % filename
                    continue
                pathname = path + '/' + filename
                lines = open(pathname).read().split("\n")
                values = [r.groups()[0] for r in [recomp.search(l) for l in lines] if r]
                if values:
                    #- Get the current english translations for the path
                    transgood = get_translations(path)
                    if not transgood:
                        print "No translation file found for %s, skipping %s" % (path, pathname)
                        continue
                    notfound = list()
                    if DEBUG:print pathname
                    for value in values:
                        #- Try to find the value from the source file in the translation file
                        escaped_value = re.escape(value)
                        found = re.search('(%s|"%s")\s*=\s*"(.*?)";' % (escaped_value, escaped_value), transgood)
                        if found:
                            if DEBUG: print "\t", '[%s] FOUND -- "%s"' % found.groups()
                        else:
                            #notfound.append("-->\t[%s] ==== Not Found ====" % value)
                            notfound.append("""-->\t"%s" = "%s";""" % (value, value))
                    if notfound:
                        if not DEBUG:print pathname
                        print "\n".join(notfound)


def main():
    #- Only one option...
    if len(sys.argv) > 1: 
        if sys.argv[1] == '-g':
            global DEBUG
            DEBUG = True
        else:
            print 'Usage:', sys.argv[0], '[-g]\n\t\t-g: debug will show matching also'
            sys.exit(1)

    #greylist = ('UIxFilterEditor.wox')
    greylist = ()

    #- Get only the label:value from all lines
    recomp = re.compile('.label:[^=]*="(.*?)"')
    find_missing_translations('../UI', 'wox', recomp, greylist)

    #- [self labelForKey: @"Issuer"]
    recomp = re.compile('\[self labelForKey: @"(.*?)"\]')
    find_missing_translations('../UI', 'm', recomp, ())

main()



#!/usr/bin/python

import sys
import MySQLdb
import webdavlib

import HordeSignatureConverter

from config import webdavConfig, dbConfig

xmlns_inversedav = "urn:inverse:params:xml:ns:inverse-dav"

def UploadSignature(client, user, signature):
    collection = '/SOGo/dav/%s/' % user
    proppatch \
        = webdavlib.WebDAVPROPPATCH(collection,
                                    { "{%s}signature" % xmlns_inversedav: \
                                          signature.encode("utf-8") })
    client.execute(proppatch)
    if (proppatch.response["status"] < 200
        or proppatch.response["status"] > 399):
        print "Failure uploading signature for user '%s': %d" \
            % (user, proppatch.response["status"])

if __name__ == "__main__":
    if len(sys.argv) > 1:
        user = sys.argv[1]
    else:
        raise Exception, "<user> argument must be specified" \
            " (use 'ALL' for everyone)"

    conn = MySQLdb.connect(host = dbConfig["hostname"],
                           user = dbConfig["username"],
                           passwd = dbConfig["password"],
                           db = dbConfig["database"],
                           use_unicode = True)

    cnv = HordeSignatureConverter.HordeSignatureConverter(user, "DOMAIN.COM")
    signatures = cnv.fetchSignatures(conn)
    conn.close()

    client = webdavlib.WebDAVClient(webdavConfig["hostname"],
                                    webdavConfig["port"],
                                    webdavConfig["username"],
                                    webdavConfig["password"])
    for user in signatures:
        signature = signatures[user]
        UploadSignature(client, user, signature)



import PHPDeserializer
import sys

class HordeSignatureConverter:
    def __init__(self, user, domain):
        self.user = user
        self.domain = domain
        self.domainLen = len(domain)

    def fetchSignatures(self, conn):
        self.signatures = None
        self.conn = conn
        self.fetchIdentities()

        return self.signatures

    def fetchIdentities(self):
        self.users = {}

        cursor = self.conn.cursor()
        if self.user == "ALL":
            userClause = ""
        else:
            userClause = "AND pref_uid = '%s'" % self.user
        query = "SELECT pref_uid, pref_value" \
            "      FROM horde_prefs" \
            "     WHERE pref_scope = 'horde'" \
            "       AND pref_name = 'identities'" \
            "        %s" % userClause
        cursor.execute(query)

        self.signatures = {}
        records = cursor.fetchall()
        max = len(records)
        if max > 0:
            for record in records:
                user = record[0]
                signature = self.decodeSignature(record[1], user)
                if signature is None or len(signature.strip()) == 0:
                    print "No useful signature found for %s" % user
                else:
                    self.signatures[user] = signature

            print "%d useful signature(s) found in %d record(s)" % (len(self.signatures), max)
        else:
            print "No record found"

        cursor.close()

    def decodeSignature(self, prefs, user):
        des = PHPDeserializer.PHPDeserializer(prefs)
        identities = des.deserialize()
        nbrEntries = len(identities)
        signatures = []
        for identity in identities:
            fromAddr = identity["from_addr"]
            if (len(fromAddr) > self.domainLen
                and fromAddr[-self.domainLen:] == self.domain):
                if identity.has_key("signature"):
                    signatures.append(identity["signature"])

        if len(signatures) > 0:
            signature = self.chooseSignature(signatures)
        else:
            signature = None

        return signature

    def chooseSignature(self, signatures):
        biggest = -1
        length = -1
        count = 0
        for signature in signatures:
            thisLength = len(signature)
            if thisLength > 0 and thisLength > length:
                biggest = count
            count = count + 1

        if biggest == -1:
            signature = None
        else:
            signature = signatures[biggest]

        return signature
